@@ -40,46 +40,23 @@ def html_escape(text):
   """Produce entities within text."""
   return "".join(html_escape_table.get(c,c) for c in text)
 
-def checkLogin(config, login, pwd):
-  """
-  User authentication method
-
-  Args:
-    config: Slaprunner configuration.
-    login: username of the user.
-    pwd: password associate to username.
-
-  Returns:
-    a list of user informations or False if authentication fail.
-    list=[username, password, email, complete_name]
-  """
-  user = getSession(config)
-  salt = "runner81" #to be changed
-  current_pwd = hashlib.md5( salt + pwd ).hexdigest()
-  if current_pwd == user[1]:
-    return user
-  return False
-
 def getSession(config):
   """
   Get the session data of current user.
   Returns:
     a list of user informations or False if fail to read data.
   """
-  user_path = os.path.join(config['runner_workdir'], '.users')
+  user_path = os.path.join(config['etc_dir'], '.users')
   user = ""
   if os.path.exists(user_path):
-    user = open(user_path, 'r').read().split(';')
+    f = open(user_path, 'r')
+    user = f.read().split(';')
+    f.close()
   if type(user) == type(""):
-    #Error: try to restore data from backup
-    if os.path.exists(user_path+'.back'):
-      os.rename(user_path+'.back', user_path)
-      user = open(user_path, 'r').read().split(';')
-    else:
-      return False
+    return False
   return user
 
-def saveSession(config, session, account):
+def saveSession(config, account):
   """
   Save account information for the current user
 
@@ -91,20 +68,20 @@ def saveSession(config, session, account):
   Returns:
     True if all goes well or str (error message) if fail
   """
-  user = os.path.join(config['runner_workdir'], '.users')
+  user = os.path.join(config['etc_dir'], '.users')
   backup = False
   try:
-    if account[1]:
-      salt = "runner81" #to be changed
-      account[1] = hashlib.md5(salt + account[1]).hexdigest()
-    else:
-      account[1] = session['account'][1]
-    #backup previous data
-    open(user+'.back', 'w').write(';'.join(session['account']))
-    backup = True
+    if os.path.exists(user):
+      f = open(user, 'r')
+      #backup previous data
+      data = f.read()
+      open(user+'.back', 'w').write(data)
+      f.close()
+      backup = True
+      if not account[1]:
+        account[1] = data.split(';')[1]
     #save new account data
     open(user, 'w').write((';'.join(account)).encode("utf-8"))
-    session['account'] = account
     return True
   except Exception, e:
     try:
